package db

import (
	"go_dreamfactory/pb"
	"log"
	"os"
	"testing"

	"github.com/liwei1dao/lego/sys/mgo"
	"github.com/stretchr/testify/assert"
	"github.com/stretchr/testify/require"
)

var db *DB

func TestMain(m *testing.M) {
	imgo, err := mgo.NewSys(mgo.SetMongodbUrl("mongodb://admin:123456@10.0.0.9:27018"), mgo.SetMongodbDatabase("dreamfactory"))
	if err != nil {
		log.Fatal(err)
	}

	db = &DB{
		mgo: imgo,
	}
	defer os.Exit(m.Run())
}

func TestCreate(t *testing.T) {
	user := &pb.DB_UserData{
		Account:  "legu3",
		NiceName: "乐谷3",
		Email:    "1111@legu.com",
	}

	err := db.CreateUser(user)
	require.Nil(t, err)
}

func TestFindOne(t *testing.T) {
	user, err := db.FindUserById(1)
	require.Nil(t, err)
	assert.Equal(t, "legu1", user.Account)

	user2, err := db.FindUserByAccount("legu1")
	require.Nil(t, err)
	assert.Equal(t, "legu1", user2.Account)

}

func TestUpdate(t *testing.T) {
	user := &pb.DB_UserData{
		UserId: 10001,
		Email:  "new@qq.com",
	}
	err := db.UpdateUser(user)
	require.Nil(t, err)

	assert.Equal(t, "new@qq.com", user.Email)
}
